<?php

namespace Teknisa\Libs\Controller;

use Teknisa\Libs\Util\Utilities;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Service\Login as LoginService;
use Teknisa\Libs\Service\Api as ApiService;

class Login {
    /** @var LoginService $loginService */
    private $loginService;
    /** @var ApiService $apiService */
    private $apiService;

    /** @param LoginService $loginService */
    /** @param ApiService $apiService */
    public function __construct(LoginService $loginService, ApiService $apiService) {
        $this->loginService = $loginService;
        $this->apiService   = $apiService;
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     * @throws \Exception
     */
    public function startSession(Request\Filter $request, Response $response) {
        try {
            $conditions   = $request->getFilterCriteria()->getConditions();
            $token        = Utilities::getValueInFilterDataSourceByName($conditions, 'TOKEN');
            $versionFile  = Utilities::getValueInFilterDataSourceByName($conditions, 'VERSION_FILE');
            $testDataBase = Utilities::getValueInFilterDataSourceByName($conditions, 'TEST_DATA_BASE');
            $productId    = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID');
            $isLogged     = Utilities::getValueInFilterDataSourceByName($conditions, 'IS_LOGGED');
            $userData     = $this->loginService->restoreSession($token, $versionFile, $testDataBase, $productId, $isLogged);
            $response->addDataSet(new DataSet('system', $userData));
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            $response->setError(new Response\Error($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     * @throws \Exception
     */
    public function logout(Request\Filter $request, Response $response) {
        try {
            $this->loginService->logout();
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            throw $e;
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     * @throws \Exception
     */
    public function updateSession(Request\Filter $request, Response $response) {
        try {
            $conditions         = $request->getFilterCriteria()->getConditions();
            $user               = Utilities::getValueInFilterDataSourceByName($conditions, 'USER');
            $accessProfile      = Utilities::getValueInFilterDataSourceByName($conditions, 'ACCESS_PROFILE');
            $otherSessionValues = Utilities::getValueInFilterDataSourceByName($conditions, 'OTHER_SESSION_VALUES');
            $host               = Utilities::getValueInFilterDataSourceByName($conditions, 'IP');
            $port               = Utilities::getValueInFilterDataSourceByName($conditions, 'PORT');
            $userSession = $this->loginService->updateSession($user, $accessProfile, $host, $port, $otherSessionValues);
            $response->addDataSet(new DataSet('sessionUpdated', $userSession));
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            $response->setError(new Response\Error($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     * @throws \Exception
     */
    public function getIpAddress(Request\Filter $request, Response $response) {
        try {
            $ip = $this->loginService->getCurrentIp();
            $bHash = $this->loginService->getBHash();
            $modulesVersionError = null;

            try{
                $this->apiService->getModulesVersion();
            } catch(\Exception $e) {
                $modulesVersionError = $e->getMessage();
            }

            $response->addDataSet(new DataSet('ip', array(
                'IP' => $ip, 
                'BHASH' => $bHash,
                'MODULES_VERSION_ERROR' => $modulesVersionError
            )));
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            $response->setError(new Response\Error($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param Request\Filter $request
     * @param Response $response
     * @throws \Exception
     */
    public function validateExpiredSession(Request\Filter $request, Response $response) {
        try {
            $conditions  = $request->getFilterCriteria()->getConditions();
            $timeout     = Utilities::getValueInFilterDataSourceByName($conditions, 'TIMEOUT');
            $sessionInfo = $this->loginService->validateExpiredSession($timeout);
            $response->addDataSet(new DataSet('expiredSessionInfo', $sessionInfo));
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
            $response->setError(new Response\Error($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
}